import {
  DndContext,
  useDraggable,
  closestCenter,
} from "@dnd-kit/core";
import React, { useEffect, useState } from "react";

const gridSize = 64;

function Token({ id, x, y, image, onDelete }) {
  const {
    attributes,
    listeners,
    setNodeRef,
    setActivatorNodeRef,
    transform,
  } = useDraggable({
    id,
    data: { x, y },
  });

  const style = {
    position: "absolute",
    left: x,
    top: y,
    width: gridSize,
    height: gridSize,
    transform: transform
      ? `translate3d(${transform.x}px, ${transform.y}px, 0)`
      : undefined,
    touchAction: "none",
    zIndex: 2,
  };

  return (
    <div ref={setNodeRef} style={style}>
      <img
        src={image}
        alt=""
        style={{ width: "100%", height: "100%", pointerEvents: "none" }}
      />
      
      {/* Drag Handle (only this is draggable) */}
      <div
        ref={setActivatorNodeRef}
        {...listeners}
        {...attributes}
        style={{
          position: "absolute",
          bottom: 0,
          right: 0,
          width: 16,
          height: 16,
          background: "rgba(0,0,0,0.4)",
          cursor: "grab",
          borderRadius: "2px",
        }}
        title="Drag"
      />

      {/*Delete button */}
      <button
        onClick={(e) => {
          e.stopPropagation();
          e.preventDefault();
          onDelete(id);
        }}
        style={{
          position: "absolute",
          top: -10,
          right: -10,
          fontSize: "10px",
          background: "red",
          color: "white",
          border: "none",
          borderRadius: "50%",
          width: 20,
          height: 20,
          cursor: "pointer",
          zIndex: 5,
        }}
        title="Delete"
      >
        ×
      </button>
    </div>
  );
}


export default function Spilleplade() {
  const [spilleplade, setSpilleplade] = useState(null);

  useEffect(() => {
    fetch("http://localhost:5000/api/spilleplade")
      .then((res) => res.json())
      .then((data) => setSpilleplade(data[0]));
  }, []);

  function addNewToken() {
    if (!spilleplade) return;
  
    const newToken = {
      name: "New Token",
      type: "obstacle",
      image: "/Terrain/rock.jpg",
      x: 0,
      y: 0,
      blocksMovement: false,
      movementCost: 1,
      spillepladeID: spilleplade.id
    };
  
    fetch("http://localhost:5000/api/terrain", {
      method: "POST",
      headers: {
        "Content-Type": "application/json",
        Accept: "application/json" // 👈 Hint to the server!
      },
      body: JSON.stringify(newToken)
    })
      .then(async (res) => {
        const contentType = res.headers.get("content-type");
        const rawText = await res.text();
  
        console.log("🧾 Raw response:", rawText);
  
        if (!res.ok) {
          throw new Error(`Failed to create token: ${res.status}`);
        }
  
        // Safely parse if JSON is returned
        if (contentType && contentType.includes("application/json")) {
          return JSON.parse(rawText);
        } else {
          return {}; // fallback if backend returned plain text
        }
      })
      .then((created) => {
        console.log("✅ Token created:", created);
  
        setSpilleplade((prev) => ({
          ...prev,
          terrain: [...prev.terrain, created]
        }));
      })
      .catch((err) => {
        console.error("❌ Failed to add token:", err);
      });
  }
  
  function DeleteTerrain(id)
  {
    fetch(`http://localhost:5000/api/terrain/${id}`, {
      method: "DELETE",
    })
      .then((res) => {
        if (!res.ok) {
          throw new Error(`Failed to delete token ${id}`);
        }
        setSpilleplade((prev) => ({
          ...prev,
          terrain: prev.terrain.filter((t) => t.id !== id),
        }));
      })
      .catch((err) => {
        console.error("❌ Failed to delete token:", err);
      });
  }

  if (!spilleplade) return <div>Indlæser spilleplade…</div>;

  return (
    <>
    <button onClick={addNewToken} style={{marginBottom: "10px"}}>
    ➕ Tilføj terræn
    </button>
    <div
      style={{
        position: "relative",
        width: 800,
        height: 800,
        border: "2px solid #444",
      }}
    >
      {/* Background image */}
      <img
        src={spilleplade.backgroundImage}
        alt="Map"
        style={{
          width: "100%",
          height: "100%",
          objectFit: "cover",
          position: "absolute",
          zIndex: 0,
        }}
      />

      {/* Grid overlay */}
      <div
        style={{
          position: "absolute",
          width: "100%",
          height: "100%",
          backgroundSize: `${spilleplade.cellsize}px ${spilleplade.cellsize}px`,
          backgroundImage:
            spilleplade.gridtype === "hexagonal"
              ? "repeating-linear-gradient(60deg, #ccc 1px, transparent 1px, transparent 20px)"
              : `linear-gradient(to right, #ccc 1px, transparent 1px),
                 linear-gradient(to bottom, #ccc 1px, transparent 1px)`,
          pointerEvents: "none",
          zIndex: 1,
        }}
      />

      {/* Draggable tokens */}
      <DndContext
        collisionDetection={closestCenter}
        onDragEnd={(event) => {
          const { active, delta } = event;
          const original = active.data.current;

          // Calculate new snapped position
          const newX = Math.round((original.x + delta.x) / spilleplade.cellsize) * spilleplade.cellsize;
          const newY = Math.round((original.y + delta.y) / spilleplade.cellsize) * spilleplade.cellsize;

          console.log(`Moved token ${active.id} to: (${newX}, ${newY})`);

          // Update local state
          setSpilleplade((prev) => ({
            ...prev,
            terrain: prev.terrain.map((t) =>
              t.id === active.id ? { ...t, x: newX, y: newY } : t
            ),
          }));

          // Save to backend
          fetch(`http://localhost:5000/api/terrain/${active.id}`, {
            method: "PUT",
            headers: { "Content-Type": "application/json" },
            body: JSON.stringify({ x: newX, y: newY }),
          }).catch((err) =>
            console.error("Failed to save token position:", err)
          );
        }}
      >
        {spilleplade.terrain.map((t) => (
          <Token key={t.id} id={t.id} x={t.x} y={t.y} image={t.image} onDelete={DeleteTerrain} />
        ))}
      </DndContext>
    </div>
    </>
  );
}
